@@ -1,4 +1,4 @@
-const char errlog_rcs[] = "$Id: errlog.c,v 1.100 2010/01/03 12:37:14 fabiankeil Exp $";
+const char errlog_rcs[] = "$Id: errlog.c,v 1.117 2012/12/09 12:28:14 fabiankeil Exp $";
 /*********************************************************************
  *
  * File        :  $Source: /cvsroot/ijbswa/current/errlog.c,v $
@@ -6,14 +6,14 @@ const char errlog_rcs[] = "$Id: errlog.c,v 1.100 2010/01/03 12:37:14 fabiankeil
  * Purpose     :  Log errors to a designated destination in an elegant,
  *                printf-like fashion.
  *
- * Copyright   :  Written by and Copyright (C) 2001-2009 the SourceForge
+ * Copyright   :  Written by and Copyright (C) 2001-2010 the
  *                Privoxy team. http://www.privoxy.org/
  *
  *                Based on the Internet Junkbuster originally written
- *                by and Copyright (C) 1997 Anonymous Coders and 
+ *                by and Copyright (C) 1997 Anonymous Coders and
  *                Junkbusters Corporation.  http://www.junkbusters.com
  *
- *                This program is free software; you can redistribute it 
+ *                This program is free software; you can redistribute it
  *                and/or modify it under the terms of the GNU General
  *                Public License as published by the Free Software
  *                Foundation; either version 2 of the License, or (at
@@ -38,6 +38,7 @@ const char errlog_rcs[] = "$Id: errlog.c,v 1.100 2010/01/03 12:37:14 fabiankeil
 #include <stdio.h>
 #include <stdarg.h>
 #include <string.h>
+#include <ctype.h>
 
 #include "config.h"
 #include "miscutil.h"
@@ -120,9 +121,9 @@ static inline void unlock_loginit(void)
 #else /* ! MUTEX_LOCKS_AVAILABLE */
 /*
  * FIXME we need a cross-platform locking mechanism.
- * The locking/unlocking functions below should be 
+ * The locking/unlocking functions below should be
  * fleshed out for non-pthread implementations.
- */ 
+ */
 static inline void lock_logfile() {}
 static inline void unlock_logfile() {}
 static inline void lock_loginit() {}
@@ -133,7 +134,7 @@ static inline void unlock_loginit() {}
  *
  * Function    :  fatal_error
  *
- * Description :  Displays a fatal error to standard error (or, on 
+ * Description :  Displays a fatal error to standard error (or, on
  *                a WIN32 GUI, to a dialog box), and exits Privoxy
  *                with status code 1.
  *
@@ -145,26 +146,28 @@ static inline void unlock_loginit() {}
  *********************************************************************/
 static void fatal_error(const char *error_message)
 {
-#if defined(_WIN32) && !defined(_WIN_CONSOLE)
-   /* Skip timestamp and thread id for the message box. */
-   const char *box_message = strstr(error_message, "Fatal error");
-   if (NULL == box_message)
-   {
-      /* Shouldn't happen but ... */
-      box_message = error_message;
-   }
-   MessageBox(g_hwndLogFrame, box_message, "Privoxy Error", 
-      MB_OK | MB_ICONERROR | MB_TASKMODAL | MB_SETFOREGROUND | MB_TOPMOST);  
-
-   /* Cleanup - remove taskbar icon etc. */
-   TermLogWindow();
-#endif /* defined(_WIN32) && !defined(_WIN_CONSOLE) */
-
    if (logfp != NULL)
    {
       fputs(error_message, logfp);
    }
 
+#if defined(_WIN32) && !defined(_WIN_CONSOLE)
+   {
+      /* Skip timestamp and thread id for the message box. */
+      const char *box_message = strstr(error_message, "Fatal error");
+      if (NULL == box_message)
+      {
+         /* Shouldn't happen but ... */
+         box_message = error_message;
+      }
+      MessageBox(g_hwndLogFrame, box_message, "Privoxy Error",
+         MB_OK | MB_ICONERROR | MB_TASKMODAL | MB_SETFOREGROUND | MB_TOPMOST);
+
+      /* Cleanup - remove taskbar icon etc. */
+      TermLogWindow();
+   }
+#endif /* defined(_WIN32) && !defined(_WIN_CONSOLE) */
+
 #if defined(unix)
    if (pidfile)
    {
@@ -231,7 +234,7 @@ void init_log_module(void)
  *                XXX: we should only use the LOG_LEVEL_MINIMUM
  *                until the first time the configuration file has
  *                been parsed.
- *                
+ *
  * Parameters  :  1: debug_level = The debug level to set.
  *
  * Returns     :  Nothing.
@@ -243,12 +246,29 @@ void set_debug_level(int debug_level)
 }
 
 
+/*********************************************************************
+ *
+ * Function    :  debug_level_is_enabled
+ *
+ * Description :  Checks if a certain debug level is enabled.
+ *
+ * Parameters  :  1: debug_level = The debug level to check.
+ *
+ * Returns     :  Nothing.
+ *
+ *********************************************************************/
+int debug_level_is_enabled(int debug_level)
+{
+   return (0 != (debug & debug_level));
+}
+
+
 /*********************************************************************
  *
  * Function    :  disable_logging
  *
  * Description :  Disables logging.
- *                
+ *
  * Parameters  :  None.
  *
  * Returns     :  Nothing.
@@ -316,7 +336,7 @@ void init_error_log(const char *prog_name, const char *logfname)
        * Currently we reopen it every time the config file
        * has been reloaded, but actually we only have to
        * reopen it if the file name changed or if the
-       * configuration reloas was caused by a SIGHUP.
+       * configuration reload was caused by a SIGHUP.
        */
       log_error(LOG_LEVEL_INFO, "Failed to reopen logfile: \'%s\'. "
          "Retrying after closing the old file descriptor first. If that "
@@ -374,7 +394,7 @@ void init_error_log(const char *prog_name, const char *logfname)
  * Description :  Returns a number that is different for each thread.
  *
  *                XXX: Should be moved elsewhere (miscutil.c?)
- *                
+ *
  * Parameters  :  None
  *
  * Returns     :  thread_id
@@ -429,7 +449,7 @@ static long get_thread_id(void)
 static inline size_t get_log_timestamp(char *buffer, size_t buffer_size)
 {
    size_t length;
-   time_t now; 
+   time_t now;
    struct tm tm_now;
    struct timeval tv_now; /* XXX: stupid name */
    long msecs;
@@ -443,16 +463,16 @@ static inline size_t get_log_timestamp(char *buffer, size_t buffer_size)
    tm_now = *localtime_r(&now, &tm_now);
 #elif defined(MUTEX_LOCKS_AVAILABLE)
    privoxy_mutex_lock(&localtime_mutex);
-   tm_now = *localtime(&now); 
+   tm_now = *localtime(&now);
    privoxy_mutex_unlock(&localtime_mutex);
 #else
-   tm_now = *localtime(&now); 
+   tm_now = *localtime(&now);
 #endif
 
-   length = strftime(buffer, buffer_size, "%b %d %H:%M:%S", &tm_now);
+   length = strftime(buffer, buffer_size, "%Y-%m-%d %H:%M:%S", &tm_now);
    if (length > (size_t)0)
    {
-      msecs_length = snprintf(buffer+length, buffer_size - length, ".%.3ld", msecs);               
+      msecs_length = snprintf(buffer+length, buffer_size - length, ".%.3ld", msecs);
    }
    if (msecs_length > 0)
    {
@@ -487,7 +507,7 @@ static inline size_t get_clf_timestamp(char *buffer, size_t buffer_size)
     * the %z field in strftime()
     */
    time_t now;
-   struct tm *tm_now; 
+   struct tm *tm_now;
    struct tm gmt;
 #ifdef HAVE_LOCALTIME_R
    struct tm dummy;
@@ -496,7 +516,7 @@ static inline size_t get_clf_timestamp(char *buffer, size_t buffer_size)
    size_t length;
    int tz_length = 0;
 
-   time (&now); 
+   time (&now);
 #ifdef HAVE_GMTIME_R
    gmt = *gmtime_r(&now, &gmt);
 #elif defined(MUTEX_LOCKS_AVAILABLE)
@@ -510,14 +530,14 @@ static inline size_t get_clf_timestamp(char *buffer, size_t buffer_size)
    tm_now = localtime_r(&now, &dummy);
 #elif defined(MUTEX_LOCKS_AVAILABLE)
    privoxy_mutex_lock(&localtime_mutex);
-   tm_now = localtime(&now); 
+   tm_now = localtime(&now);
    privoxy_mutex_unlock(&localtime_mutex);
 #else
-   tm_now = localtime(&now); 
+   tm_now = localtime(&now);
 #endif
-   days = tm_now->tm_yday - gmt.tm_yday; 
-   hrs = ((days < -1 ? 24 : 1 < days ? -24 : days * 24) + tm_now->tm_hour - gmt.tm_hour); 
-   mins = hrs * 60 + tm_now->tm_min - gmt.tm_min; 
+   days = tm_now->tm_yday - gmt.tm_yday;
+   hrs = ((days < -1 ? 24 : 1 < days ? -24 : days * 24) + tm_now->tm_hour - gmt.tm_hour);
+   mins = hrs * 60 + tm_now->tm_min - gmt.tm_min;
 
    length = strftime(buffer, buffer_size, "%d/%b/%Y:%H:%M:%S ", tm_now);
 
@@ -545,7 +565,7 @@ static inline size_t get_clf_timestamp(char *buffer, size_t buffer_size)
  *
  * Description :  Translates a numerical loglevel into a string.
  *
- * Parameters  :  
+ * Parameters  :
  *          1  :  loglevel = LOG_LEVEL_FOO
  *
  * Returns     :  Log level string.
@@ -571,9 +591,12 @@ static inline const char *get_log_level_string(int loglevel)
       case LOG_LEVEL_CONNECT:
          log_level_string = "Connect";
          break;
-      case LOG_LEVEL_LOG:
+      case LOG_LEVEL_WRITING:
          log_level_string = "Writing";
          break;
+      case LOG_LEVEL_RECEIVED:
+         log_level_string = "Received";
+         break;
       case LOG_LEVEL_HEADER:
          log_level_string = "Header";
          break;
@@ -600,6 +623,9 @@ static inline const char *get_log_level_string(int loglevel)
       case LOG_LEVEL_CGI:
          log_level_string = "CGI";
          break;
+      case LOG_LEVEL_ACTIONS:
+         log_level_string = "Actions";
+         break;
       default:
          log_level_string = "Unknown log level";
          break;
@@ -678,7 +704,7 @@ void log_error(int loglevel, const char *fmt, ...)
    /* protect the whole function because of the static buffer (outbuf) */
    lock_logfile();
 
-   if (NULL == outbuf_save) 
+   if (NULL == outbuf_save)
    {
       outbuf_save = (char*)zalloc(log_buffer_size + 1); /* +1 for paranoia */
       if (NULL == outbuf_save)
@@ -739,11 +765,11 @@ void log_error(int loglevel, const char *fmt, ...)
             tempbuf[1] = '\0';
             break;
          case 'd':
-            ival = va_arg( ap, int );
+            ival = va_arg(ap, int);
             snprintf(tempbuf, sizeof(tempbuf), "%d", ival);
             break;
          case 'u':
-            uval = va_arg( ap, unsigned );
+            uval = va_arg(ap, unsigned);
             snprintf(tempbuf, sizeof(tempbuf), "%u", uval);
             break;
          case 'l':
@@ -751,12 +777,12 @@ void log_error(int loglevel, const char *fmt, ...)
             ch = *src++;
             if (ch == 'd')
             {
-               lval = va_arg( ap, long );
+               lval = va_arg(ap, long);
                snprintf(tempbuf, sizeof(tempbuf), "%ld", lval);
             }
             else if (ch == 'u')
             {
-               ulval = va_arg( ap, unsigned long );
+               ulval = va_arg(ap, unsigned long);
                snprintf(tempbuf, sizeof(tempbuf), "%lu", ulval);
             }
             else if ((ch == 'l') && (*src == 'u'))
@@ -788,45 +814,36 @@ void log_error(int loglevel, const char *fmt, ...)
             break;
          case 'N':
             /*
-             * Non-standard: Print a counted unterminated string.
+             * Non-standard: Print a counted unterminated string,
+             * replacing unprintable bytes with their hex value.
              * Takes 2 parameters: int length, const char * string.
              */
             ival = va_arg(ap, int);
+            assert(ival >= 0);
             sval = va_arg(ap, char *);
-            if (sval == NULL)
+            assert(sval != NULL);
+
+            while ((ival-- > 0) && (length < log_buffer_size - 6))
             {
-               format_string = "[null]";
-            }
-            else if (ival <= 0)
-            {
-               if (0 == ival)
+               if (isprint((int)*sval) && (*sval != '\\'))
                {
-                  /* That's ok (but stupid) */
-                  tempbuf[0] = '\0';
+                  outbuf[length++] = *sval;
+                  outbuf[length] = '\0';
                }
                else
                {
-                  /*
-                   * That's not ok (and even more stupid)
-                   */
-                  assert(ival >= 0);
-                  format_string = "[counted string lenght < 0]";
+                  int ret = snprintf(outbuf + length,
+                     log_buffer_size - length - 2, "\\x%.2x", (unsigned char)*sval);
+                  assert(ret == 4);
+                  length += 4;
                }
+               sval++;
             }
-            else if ((size_t)ival >= sizeof(tempbuf))
-            {
-               /*
-                * String is too long, copy as much as possible.
-                * It will be further truncated later.
-                */
-               memcpy(tempbuf, sval, sizeof(tempbuf)-1);
-               tempbuf[sizeof(tempbuf)-1] = '\0';
-            }
-            else
-            {
-               memcpy(tempbuf, sval, (size_t) ival);
-               tempbuf[ival] = '\0';
-            }
+            /*
+             * XXX: In case of printable characters at the end of
+             *      the %N string, we're not using the whole buffer.
+             */
+            format_string = (length < log_buffer_size - 6) ? "" : "[too long]";
             break;
          case 'E':
             /* Non-standard: Print error code from errno */
@@ -845,7 +862,7 @@ void log_error(int loglevel, const char *fmt, ...)
                format_string = strerror(ival);
             }
 #else /* ifndef _WIN32 */
-            ival = errno; 
+            ival = errno;
 #ifdef HAVE_STRERROR
             format_string = strerror(ival);
 #else /* ifndef HAVE_STRERROR */
@@ -865,14 +882,14 @@ void log_error(int loglevel, const char *fmt, ...)
             snprintf(tempbuf, sizeof(tempbuf), "Bad format string: \"%s\"", fmt);
             loglevel = LOG_LEVEL_FATAL;
             break;
-      } /* switch( p ) */
+      }
 
       assert(length < log_buffer_size);
       length += strlcpy(outbuf + length, format_string, log_buffer_size - length);
 
       if (length >= log_buffer_size-2)
       {
-         static char warning[] = "... [too long, truncated]";
+         static const char warning[] = "... [too long, truncated]";
 
          length = log_buffer_size - sizeof(warning) - 1;
          length += strlcpy(outbuf + length, warning, log_buffer_size - length);
@@ -880,7 +897,7 @@ void log_error(int loglevel, const char *fmt, ...)
 
          break;
       }
-   } /* for( p ... ) */
+   }
 
    /* done with var. args */
    va_end(ap);
@@ -950,14 +967,14 @@ void log_error(int loglevel, const char *fmt, ...)
  *                visible to all files that include errlog.h.
  *
  * Parameters  :
- *          1  :  error = a valid jb_err code
+ *          1  :  jb_error = a valid jb_err code
  *
  * Returns     :  A string with the jb_err translation
  *
  *********************************************************************/
-const char *jb_err_to_string(int error)
+const char *jb_err_to_string(int jb_error)
 {
-   switch (error)
+   switch (jb_error)
    {
       case JB_ERR_OK:
          return "Success, no error";
